// counter.c $Id: 3b0ed3f9a56b2a5259a028b13e0a012f973d4288 $

#include <sys/time.h>

/* If compiled e.g. with -ansi, timersub is not defined. In this case, re-define
   it here (copied from sys/time.h). */

#ifndef timersub
# define timersub(a, b, result)						      \
  do {									      \
    (result)->tv_sec = (a)->tv_sec - (b)->tv_sec;			      \
    (result)->tv_usec = (a)->tv_usec - (b)->tv_usec;			      \
    if ((result)->tv_usec < 0) {					      \
      --(result)->tv_sec;						      \
      (result)->tv_usec += 1000000;					      \
    }									      \
  } while (0)
#endif

#include "pciaer.h"
#include "pciaerlib.h"

/******************************************************************************/

/* Internal copy of the current PCI-AER internal clock period, fetched from
   driver by PciaerGetCounterPeriodFromDriver whenever Monitor or Sequencer is
   opened and changed whenever PciaerSetCounterPeriod is called. */

static int CurrentClockPeriod_us = 1;

/******************************************************************************/

int PciaerGetCounterPeriodFromDriver(int handle)
{
    return (ioctl(handle,PCIAER_IOC_GET_CNTR_PERIOD,&CurrentClockPeriod_us) == 0) ? 0 : errno;
}

/******************************************************************************/

int PciaerSetCounterPeriod(int handle, int Period_us)
{
    if ( ioctl(handle,PCIAER_IOC_SET_CNTR_PERIOD,Period_us) == 0 )
	{
	    CurrentClockPeriod_us = Period_us;
	    return 0;
	}
    else
	return errno;
}

/******************************************************************************/

int PciaerGetCounterPeriodLocalCopy( void )
{
    return CurrentClockPeriod_us;
}

/******************************************************************************/

int PciaerGetCounterPeriod(int handle, int *pPeriod_us)
{
    *pPeriod_us = CurrentClockPeriod_us;
    return 0;
}

/******************************************************************************/

int PciaerGetCounterValue(int handle, unsigned int *pValue)
{
    struct timeval ResetTime;
    struct timeval TimeNow;
    struct timeval ElapsedTime;

    if ( ioctl( handle, PCIAER_IOC_GET_CNTR_LAST_RST_TIME, &ResetTime ) != 0 )
	return errno;
    if ( gettimeofday( &TimeNow, NULL ) != 0 )
	return errno;
    timersub( &TimeNow, &ResetTime, &ElapsedTime );
    *pValue = (unsigned int)(((long long)ElapsedTime.tv_sec * 1000000 + ElapsedTime.tv_usec) / CurrentClockPeriod_us);
    return 0;
}

/******************************************************************************/

/* EOF */
