/* monopen.c $Id: 6b05581a5ced3650cd1540b69c975175f90723f2 $ */

#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>

#include "pciaer.h"
#include "pciaerlib.h"

extern int PciaerGetCounterPeriodFromDriver( int handle );

/******************************************************************************/

/* Buffer in which events (or parts of events in the case that time labels are
// enabled) may be preserved between invocations of PciaerMonRead.
// Malloc'd/realloc'd in PciaerMonRead, free'd in PciaerMonClose.
*/

int *pRawWordsBuffer = NULL;

/******************************************************************************/

/* PciaerMonOpen 

// Opens the monitor sub-device on the given PCI-AER board, indexed from 0.
// The flags are the same as those that may be supplied to the system call open;
// the only relevant possibilities here are O_RDONLY, O_WRONLY, O_RDWR, and
// O_NONBLOCK. Each board's monitor sub-device may only be opened by a single
// process to ensure that multiple processes cannot read from it simultaneously
// thus erroneously splitting the incoming data into multiple streams. (This is
// enforced by the driver.) If the device is already open, PciaerMonOpen will
// return EBUSY. Note that when the monitor device is opened, the monitor FIFO
// may contain stale, possibly very stale data. Applications must call
// PciaerResetFifo if they want to be sure of reading recent data. Opening the
// device will never block.
// Iff the open is successful (i.e. when the return value is 0) a handle to the
// monitor sub-device is returned in the integer pointed to by pHandle.
*/

#define BASE_MON_DEVICE_PATH	"/dev/aermon"

int PciaerMonOpen(unsigned int board, int flags, int *pHandle)
{
  char pathname[ sizeof(BASE_MON_DEVICE_PATH) + 2 ]; /* Length of BASE_MON_DEVICE_PATH + one digit + \0 */
  int open_return;

  /* Open requires that board is less than 10 so that we can guarantee its
     string representation doesn't overflow the pathname buffer. */

  if ( board > 9 )
      return EINVAL;

  if ( pHandle == NULL )
	return EINVAL;

  sprintf( pathname, BASE_MON_DEVICE_PATH "%d", board );

  open_return = open( pathname, flags | O_NOCTTY );

  if (open_return < 0)
	return errno;

  *pHandle = open_return;
  
  /* Update the library's internal copy of the current PCI-AER board clock period */
  PciaerGetCounterPeriodFromDriver(*pHandle);
  return 0;
}

/******************************************************************************/

int PciaerMonClose(int handle)
{
  free( pRawWordsBuffer );
  pRawWordsBuffer = NULL;
  return (close( handle ) == 0) ? 0 : errno;
}

/******************************************************************************/

/* EOF */
