/* monread.c $Id: a2704ef84c0b2bf72b848854e5089b07d768b9a5 $ */

#include <stdlib.h>
#include <string.h>

#include "pciaer.h"
#include "pciaerlib.h"

/******************************************************************************/

extern int *pRawWordsBuffer;

/******************************************************************************/

long PciaerMonRead(int handle, pciaer_monitor_read_ae_t *pUserBuffer, unsigned int nEventsToRead, unsigned int *pnEventsRead)
{
    static size_t RawWordsBufferSize = 0; /* in bytes */
  static unsigned int nValidWordsInBuffer = 0;

  int fTimeLabelsEnabled;
  unsigned int nWordsToReadRaw;
  size_t RequestedBufferSize;	/* in bytes */
  void* realloc_return;
  unsigned int nRawWordsRead;
  int read_return;
  long cook_return;
  unsigned int nRawWordsConsumed;


  /* Determine current state of time labels flag and calculate the number of
     words required for that number of events. */

  PciaerMonGetTimeLabelFlag( handle, &fTimeLabelsEnabled );
  
  nWordsToReadRaw = (fTimeLabelsEnabled ? 3 : 1) * nEventsToRead;


  /* Allocate the buffer for the raw words read from the device, or reallocate
     it in the case that a currently allocated buffer is too small. */

  if ( pRawWordsBuffer == NULL )	/* buffer not yet allocated, or was free'd by Close. */
      {
	  RawWordsBufferSize = 0;
	  nValidWordsInBuffer = 0;

	  RequestedBufferSize = nWordsToReadRaw * sizeof(int);
	  pRawWordsBuffer = (int *)malloc(RequestedBufferSize);
	  if ( pRawWordsBuffer == NULL )
	      {
#ifdef DEBUG
		  perror("Memory allocation error\n");
#endif
          *pnEventsRead = 0;
		  return ENOMEM;
	      }
	  RawWordsBufferSize = RequestedBufferSize;
      }
  else if ( RawWordsBufferSize < nWordsToReadRaw * sizeof(int) )	/* buffer too small */
      {
	  RequestedBufferSize = nWordsToReadRaw * sizeof(int);
	  realloc_return = realloc( pRawWordsBuffer, RequestedBufferSize );
	  if( realloc_return == NULL )
	      {
		  /* Old buffer pointer, size and contents are still valid and
		     can be used, providing we chop back the number of raw words
		     to read to fit. */

		  nWordsToReadRaw = RawWordsBufferSize / sizeof(int);
	      }
	  else
	      {
		  pRawWordsBuffer = (int *)realloc_return;
		  RawWordsBufferSize = RequestedBufferSize;
	      }
	}

  /* At this point we have a buffer at pRawWordsBuffer of size RawWordsBufferSize bytes
     in which we want to have nWordsToReadRaw raw words, and which already contains
     nValidWordsInBuffer raw words. */

  /* Now to fill that buffer with words from the device: */
   
  while( nValidWordsInBuffer < nWordsToReadRaw )
      {
	  read_return = PciaerMonReadRaw( handle, pRawWordsBuffer + nValidWordsInBuffer, nWordsToReadRaw - nValidWordsInBuffer, &nRawWordsRead );

	  nValidWordsInBuffer += nRawWordsRead;

	  /* In non-blocking mode, we should not go round the loop again when
	     there are no words available. */

	  if ( read_return == EAGAIN )
	      break;

	  /* In the case of other errors, return tbem to the caller. */

	  if ( read_return != 0 )
          {
              *pnEventsRead = 0;
	          return read_return;
          }
      }


  /* Now convert as many as possible words from the raw words buffer into events in the user's buffer */

  if ( fTimeLabelsEnabled )
      cook_return = CookWithTimeLabels( pRawWordsBuffer, nValidWordsInBuffer, pUserBuffer, nEventsToRead, &nRawWordsConsumed, pnEventsRead );
  else
      cook_return = CookWithoutTimeLabels( pRawWordsBuffer, nValidWordsInBuffer, pUserBuffer, nEventsToRead, &nRawWordsConsumed, pnEventsRead );

  nValidWordsInBuffer -= nRawWordsConsumed;


  /* If there are any valid words left, shift them back to the beginning of the
     buffer ready for next time round. */

  if ( nValidWordsInBuffer != 0 )
      memmove( pRawWordsBuffer, pRawWordsBuffer + nRawWordsConsumed, nValidWordsInBuffer * sizeof(int) );


  return cook_return;
}

/******************************************************************************/

/* EOF */
