/* pciaerlib.h $Id: ed82decd2e1019b4580db34391c4f43a8358434e $ */

#ifndef PCIAERLIB_H
#define PCIAERLIB_H

#include <errno.h>
#include <unistd.h>
#include <sys/ioctl.h>

/***********/
/* MONITOR */
/***********/

typedef struct{
  unsigned int ae;
  unsigned int time_us;
} pciaer_monitor_read_ae_t;

int PciaerMonOpen(unsigned int iBoard, int flags, int *pHandle);
int PciaerMonClose(int handle);
int PciaerMonReadRaw(int handle, int *p, unsigned int nToRead, unsigned int *pnRead);
long PciaerMonRead(int handle, pciaer_monitor_read_ae_t *p, unsigned int nToRead, unsigned int *pnRead);
long CookWithTimeLabels(const int *pRaw, unsigned int nRaw, pciaer_monitor_read_ae_t *pCooked, unsigned int nToCook, unsigned int *pnUsedRaw, unsigned int *pnCooked);
long CookWithoutTimeLabels(const int *pRaw, unsigned int nRaw, pciaer_monitor_read_ae_t *pCooked, unsigned int nToCook, unsigned int *pnUsedRaw, unsigned int *pnCooked);

#define PCOLERR_UNEXPECTED_TIME_HI	-2
#define PCOLERR_UNEXPECTED_TIME_LO	-3
#define PCOLERR_MISSING_TIME_LBLS	-4
#define PCOLERR_MISSING_TIME_LO 	-5

static __inline__ int PciaerMonSetChannelSel(int handle, int ch)
{
  return (ioctl(handle,PCIAER_IOC_SET_MON_CH_SEL,ch) == 0) ? 0 : errno;
}

static __inline__ int PciaerMonGetChannelSel(int handle, int *ch)
{
  return (ioctl(handle,PCIAER_IOC_GET_MON_CH_SEL,ch) == 0) ? 0 : errno;
}

static __inline__ int PciaerMonSetTimeLabelFlag(int handle, int fTimeLabelsEnabled)
{
  return (ioctl(handle,PCIAER_IOC_SET_MON_TIME_LBL_FLAG,fTimeLabelsEnabled) == 0) ? 0 : errno;
}

static __inline__ int PciaerMonGetTimeLabelFlag(int handle, int *pfTimeLabelsEnabled)
{
  return (ioctl(handle,PCIAER_IOC_GET_MON_TIME_LBL_FLAG,pfTimeLabelsEnabled) == 0) ? 0 : errno;
}

static __inline__ int PciaerMonGetFifoFlags(int handle, int *pFlags)
{
  return (ioctl(handle,PCIAER_IOC_GET_MON_FIFO_FLAGS,pFlags) == 0) ? 0 : errno;
}

/*************/
/* SEQUENCER */
/*************/

typedef struct{
  unsigned int isi_us;
  unsigned int ae;
} pciaer_sequencer_write_ae_t;

int PciaerSeqOpen(unsigned int iBoard, int flags, int *pHandle);
int PrepareRawWriteBuffer( const pciaer_sequencer_write_ae_t *pEvents, unsigned int nEvents,
                           unsigned int *pRawSeqWordsBuffer, unsigned int nRawSeqBufferWords,
                           unsigned int *pnEventsConverted, unsigned int *pnRawSeqBufferWordsUsed);
int PciaerSeqWriteRaw(int handle, const unsigned int *p, unsigned int nToWrite, unsigned int *pnWritten);
int PciaerSeqWrite(int handle, const pciaer_sequencer_write_ae_t *p, unsigned int nToWrite, unsigned int *pnWritten);

static __inline__ int PciaerSeqClose(int handle)
{
  return (close(handle) == 0) ? 0 : errno;
}

#ifndef __STRICT_ANSI__
static __inline__ int PciaerSeqFlush(int handle)
{
  return (fsync(handle) == 0) ? 0 : errno;
}
#endif

static __inline__ int PciaerSeqGetFifoFlags(int handle, int *pFlags)
{
  return (ioctl(handle,PCIAER_IOC_GET_SEQ_FIFO_FLAGS,pFlags) == 0) ? 0 : errno;
}


/**********/
/* MAPPER */
/**********/

int PciaerMapOpen(unsigned int iBoard, int flags, int *pHandle );
const volatile pciaer_mapper_sram_t * PciaerMapGetMapperMemory(int handle);
int PciaerMapSetMapping(int handle, unsigned short source, unsigned short count, const unsigned short *pDestList);
int PciaerMapGetMappingCount(int handle, unsigned short source, unsigned short *pCount);
int PciaerMapGetMapping(int handle, unsigned short source, unsigned short bufsize, unsigned short *pBuffer, unsigned short *pCount);
int PciaerMapAddToMapping(int handle, unsigned short source, unsigned short count, const unsigned short *pDestList);
int PciaerMapDeleteFromMapping(int handle, unsigned short source, unsigned short count, const unsigned short *pDestList);
int PciaerMapGetFifoFillCount( int handle, unsigned long *pc );

static __inline__ int PciaerMapClose( int handle )
{
  return close( handle ) ? errno : 0;
}

static __inline__ int PciaerMapFindNextMapping(int handle, unsigned short *pSource)
{
  return ioctl( handle, PCIAER_IOC_FIND_NEXT_MAP_MAPPING, pSource );
}

static __inline__ int PciaerMapGetMappingsBitVector(int handle, void *p)
{
  return ioctl( handle, PCIAER_IOC_GET_MAP_MAPPED_MAP, p );
}

static __inline__ int PciaerMapClearMapping(int handle, unsigned short source)
{
  return PciaerMapSetMapping( handle, source, 0, NULL );
}

static __inline__ int PciaerMapClearAllMappings( int handle )
{
  return ioctl( handle, PCIAER_IOC_MAP_CLEAR ) ? errno : 0;
}

static __inline__ int PciaerMapCompact( int handle )
{
  return ioctl( handle, PCIAER_IOC_MAP_COMPACT ) ? errno : 0;
}

static __inline__ int PciaerMapOutputEnable( int handle )
{
  return ioctl( handle, PCIAER_IOC_ENABLE_MAP_OUT ) ? errno : 0;
}

static __inline__ int PciaerMapOutputDisable( int handle, int fWait )
{
  return ioctl( handle, PCIAER_IOC_DISABLE_MAP_OUT, fWait ) ? errno : 0;
}

static __inline__ int PciaerMapGetOutputState( int handle, int *pfState )
{
  return ioctl( handle, PCIAER_IOC_GET_MAP_OUT_STATE, pfState ) ? errno : 0;
}

static __inline__ int PciaerMapSetChannelSel( int handle, int fChannels )
{
  return ioctl( handle, PCIAER_IOC_SET_MAP_CH_SEL, fChannels ) ? errno : 0;
}

static __inline__ int PciaerMapGetChannelSel( int handle, int *pfChannels )
{
  return ioctl( handle, PCIAER_IOC_GET_MAP_CH_SEL, pfChannels ) ? errno : 0;
}

static __inline__ int PciaerMapGetFifoFlags( int handle, int *pf )
{
  return ioctl( handle, PCIAER_IOC_GET_MAP_FIFO_FLAGS, pf ) ? errno : 0;
}

static __inline__ int PciaerMapSetOutputConfig( int handle, int iCfg )
{
  return ioctl( handle, PCIAER_IOC_SET_MAP_OUT_CONFIG, iCfg ) ? errno : 0;
}

static __inline__ int PciaerMapGetOutputConfig( int handle, int *piCfg )
{
  return ioctl( handle, PCIAER_IOC_GET_MAP_OUT_CONFIG, piCfg ) ? errno : 0;
}

static __inline__ int PciaerMapSetDemuxConfig( int handle, int nCfg )
{
  return ioctl( handle, PCIAER_IOC_SET_MAP_DEMUX_CONFIG, nCfg ) ? errno : 0;
}

static __inline__ int PciaerMapGetDemuxConfig( int handle, int *pnCfg )
{
  return ioctl( handle, PCIAER_IOC_GET_MAP_DEMUX_CONFIG, pnCfg ) ? errno : 0;
}

static __inline__ int PciaerMapSetProtocol( int handle, int nProtocol )
{
  return ioctl( handle, PCIAER_IOC_SET_MAP_AER_PROTOCOL, nProtocol ) ? errno : 0;
}

static __inline__ int PciaerMapGetProtocol( int handle, int *pnProtocol )
{
  return ioctl( handle, PCIAER_IOC_GET_MAP_AER_PROTOCOL, pnProtocol ) ? errno : 0;
}


/****************/
/* COMMON IOCTL */
/****************/

static __inline__ int PciaerGetVersionInfo(int handle, pciaer_version_info_t *pvi)
{
  return (ioctl(handle,PCIAER_IOC_GET_VERSION_INFO,pvi) == 0) ? 0 : errno;
}

typedef struct {
	unsigned int bus;
	char *slot_name;
	char *device_name;
	unsigned short vendor;
	unsigned short device;
	unsigned short subsys_vendor;
	unsigned short subsys_device;
	unsigned char base_class;
	unsigned char sub_class;
	unsigned char prog_if;
	unsigned char irq;
	unsigned char slot;
	unsigned char func;
} pciaer_pci_info_t;

int PciaerGetPciInfo(int handle, pciaer_pci_info_t *ppi);
int PciaerSetCounterPeriod(int handle, int period_us);
int PciaerGetCounterPeriod(int handle, int *pPeriod_us);
int PciaerGetCounterValue(int handle, unsigned int *pValue);

static __inline__ int PciaerResetCounter(int handle)
{
    return (ioctl(handle,PCIAER_IOC_RST_CNTR_GET_TIME,NULL) == 0) ? 0 : errno;
}

static __inline__ int PciaerResetCounterGetTime(int handle, struct timeval *ptv)
{
    return (ioctl(handle,PCIAER_IOC_RST_CNTR_GET_TIME,ptv) == 0) ? 0 : errno;
}

static __inline__ int PciaerGetLastCounterResetTime(int handle, struct timeval *ptv)
{
    return (ioctl(handle,PCIAER_IOC_GET_CNTR_LAST_RST_TIME,ptv) == 0) ? 0 : errno;
}

static __inline__ int PciaerSetArbConfig(int handle, int arbconf)
{
  return (ioctl(handle,PCIAER_IOC_SET_ARB_CONFIG, arbconf) == 0) ? 0 : errno;
}

static __inline__ int PciaerGetArbConfig(int handle, int *arbconf)
{
  return (ioctl(handle,PCIAER_IOC_GET_ARB_CONFIG, arbconf) == 0) ? 0 : errno;
}

static __inline__ int PciaerSetSeqArbChannel(int handle, int ch)
{
  return (ioctl(handle,PCIAER_IOC_SET_SEQ_ARB_CHANNEL,ch) == 0) ? 0 : errno;
}

static __inline__ int PciaerGetSeqArbChannel(int handle, int *ch)
{
  return (ioctl(handle,PCIAER_IOC_GET_SEQ_ARB_CHANNEL,ch) == 0) ? 0 : errno;
}

int PciaerGetFifoDepth(int handle, int *pDepth);

static __inline__ int PciaerResetFifo(int handle)
{
  return (ioctl(handle,PCIAER_IOC_RST_FIFO) == 0) ? 0 : errno;
}

int PciaerGetStatistics( int handle, pciaer_stats_t *pstats );
int PciaerResetStatistics( int handle, pciaer_stats_t *pstats );

#endif
