/* seqprep.c $Id: e115f58e7e8131697dee4df4a3ab01ff14eccfb8 $ */

#include "pciaer.h"
#include "pciaerlib.h"

extern int PciaerGetCounterPeriodLocalCopy( void );

/******************************************************************************/

/* This is the longest day that can be written as a single sequencer command */
#define SEQ_LONGEST_DELAY               ((unsigned int) SEQUENCER_DWORD_DATA_MASK)

/******************************************************************************/

int PrepareRawWriteBuffer( const pciaer_sequencer_write_ae_t *pFormattedEvents, unsigned int nNumEventsToPrepare,
                        unsigned int *pEmptyRawBuffer, unsigned int nBufferElements,
                        unsigned int *pnNumEventsPrepared, unsigned int *pnUsedBufferElements)
{
    int nClockPeriod_us;
    unsigned int    iEvent;             /* Index into formatted event buffer */
    unsigned int    iRawWord;           /* Index into raw buffer */
    unsigned int    nRemainingDelay;    /* Number of delay cycles required for a particular event */
    unsigned int    nWordsCurrentEvent; /* Number of words used by the current event */

    nClockPeriod_us = PciaerGetCounterPeriodLocalCopy();

    /* -- Initially the buffer is empty */
    *pnUsedBufferElements = 0;
    *pnNumEventsPrepared = 0;

    /* -- Write events into the buffer */
    iEvent = 0;
    iRawWord = 0;
    while (iEvent < nNumEventsToPrepare) {      /* One pass through the loop == one event */
	   nWordsCurrentEvent = 0;
	   nRemainingDelay = (pFormattedEvents[ iEvent ].isi_us + (nClockPeriod_us/2)) / nClockPeriod_us;

	   /* -- Check if the requested inter-spike interval is too long for a single-command delay */
        while (nRemainingDelay > SEQ_LONGEST_DELAY) {
            /* -- Write the longest delay event we can */
            /* If the buffer is full, then exit with no error */
            if (iRawWord >= nBufferElements)    return 0;
            
            pEmptyRawBuffer[ iRawWord++ ] = SEQ_LONGEST_DELAY | SEQUENCER_DWORD_DELAY;
            nWordsCurrentEvent++;   /* We used an extra word for this event */
            
            nRemainingDelay -= SEQ_LONGEST_DELAY;
        }

        /* -- Write a delay command for the remaining time */
        if (nRemainingDelay > 0) {
            /* If the buffer is full, then exit with no error */
            if (iRawWord >= nBufferElements)    return 0;
            pEmptyRawBuffer[ iRawWord++ ] = nRemainingDelay | SEQUENCER_DWORD_DELAY;
            nWordsCurrentEvent++;   /* We used an extra word for this event */
        }
            
	/* -- Write the spike address command */
	/* If the buffer is full, then exit with no error */
	    if (iRawWord >= nBufferElements)   return 0;
	    pEmptyRawBuffer[ iRawWord++ ] = pFormattedEvents[ iEvent ].ae | SEQUENCER_DWORD_AER_ADDR;
	    nWordsCurrentEvent++;  /* We used an extra word for this event */
	    
	    /* -- Update the number of events and number of words in the buffer */
	    iEvent++;
	    *pnNumEventsPrepared = iEvent;
	    *pnUsedBufferElements += nWordsCurrentEvent;
	}
    
    return 0;       /* No error */
}

/******************************************************************************/

/* EOF */
