// PcExpRom.cc $Id: c790a402bdf88563f659358a1caea3c5ba29e1f6 $


#include "PcExpRom.h"
#include "ReadUtil.h"


PcExpansionRomHeader::PcExpansionRomHeader( const pc_exp_rom_hdr_t *p, bool CopyReserved )
  : hdr( *p ), uhFreeSpace( CopyReserved ? 0 : sizeof(hdr.reserved) )
{
    if ( !CopyReserved )
	memset( &hdr.reserved, 0, sizeof(hdr.reserved) );
}


istream& PcExpansionRomHeader::Read( istream& is )
{
    ReadUCharArray( is, hdr.signature, sizeof(hdr.signature) );
    ReadUChar( is, hdr.length );
    ReadUCharArray( is, hdr.entry_point, sizeof(hdr.entry_point) );
    ReadUCharArray( is, hdr.reserved, sizeof(hdr.reserved) );
    uhFreeSpace = 0;
    ReadUShort( is, hdr.pci_ds_offset );
    return is;
}


bool PcExpansionRomHeader::SetEntryPoint( unsigned short uhEntryPoint )
{
    short offset = uhEntryPoint - ((hdr.entry_point - hdr.signature) + sizeof(hdr.entry_point));

    if ( offset < 0 )
	return false;

    *hdr.entry_point = PC_EXP_ROM_NEAR_JUMP;
    *reinterpret_cast<short*>(hdr.entry_point + 1) = offset;

    return true;
}


unsigned short PcExpansionRomHeader::GetEntryPoint() const
{
    if ( *hdr.entry_point != PC_EXP_ROM_NEAR_JUMP )
	return 0;
    else
	return *reinterpret_cast<const short*>(hdr.entry_point + 1) + (hdr.entry_point - hdr.signature) + sizeof(hdr.entry_point);
}


unsigned short PcExpansionRomHeader::CopyToFreeSpace( const void *p, size_t nBytes )
{
    if ( nBytes > uhFreeSpace )
	return 0;

    unsigned short offset = sizeof(hdr.reserved) - uhFreeSpace;
    memcpy( hdr.reserved + offset, p, nBytes );
    uhFreeSpace -= nBytes;

    return (hdr.reserved - hdr.signature) + offset;
}
