// PcExpRom.h $Id: b240a79787eb119edcdf7ab8b7f2ce7fd9046b49 $


#include <istream>
#include <cstring>

#include "rom.h"


using std::istream;


class PcExpansionRomHeader
{
 private:
    pc_exp_rom_hdr_t hdr;
    unsigned short uhFreeSpace; // Used for small allocations out of the header's reserved area
 public:
    PcExpansionRomHeader() : hdr(), uhFreeSpace( sizeof(hdr.reserved) ) {}
    PcExpansionRomHeader( const pc_exp_rom_hdr_t *p, bool CopyReserved );
    istream& Read( istream& is );
    unsigned char *Write( unsigned char *dest ) const { return static_cast<unsigned char *>(memcpy( dest, &hdr, sizeof(hdr) )); }
    void Sign() { hdr.signature[ 0 ] = PC_EXP_ROM_SIG1; hdr.signature[ 1 ] = PC_EXP_ROM_SIG2; }
    bool SignatureOK() const { return hdr.signature[ 0 ] == PC_EXP_ROM_SIG1 && hdr.signature[ 1 ] == PC_EXP_ROM_SIG2; }
    void SetLength( unsigned short uhBytes ) { hdr.length = (uhBytes + PC_EXP_ROM_LENGTH_MULTIPLE - 1) / PC_EXP_ROM_LENGTH_MULTIPLE; }
    unsigned short GetLength() const { return hdr.length * PC_EXP_ROM_LENGTH_MULTIPLE; }
    bool SetEntryPoint( unsigned short uhEntryPoint ); // Set near jump bytes
    unsigned short GetEntryPoint() const; // Only if near jump byte present, otherwise 0.
    unsigned short GetFreeSpace() const { return uhFreeSpace; }
    unsigned short CopyToFreeSpace( const void *p, size_t nBytes ); // Returns offset or 0 if unsuccessful.
    void SetPciDSOffset( unsigned short uhOffset ) { hdr.pci_ds_offset = uhOffset; }
    unsigned short GetPciDSOffset() const { return hdr.pci_ds_offset; }
};

inline istream& operator>>( istream& is, PcExpansionRomHeader& hdr ) { return hdr.Read( is ); }
