// PciDS.cc $Id: 697bf8b1cbfba855c5da206c281b61bb6170ebc7 $


#include "PciDS.h"
#include "ReadUtil.h"


PciDataStructure::PciDataStructure( unsigned short venid, unsigned short devid, unsigned int imglen, unsigned short rev, unsigned char codetype, unsigned char cls[ 3 ], bool last )
{
    MakeValid();
    pcids.vendor = venid;
    pcids.device = devid;
    pcids.vpd_offset = 0;
    SetClassCode( cls );
    SetImageLength( imglen );
    pcids.revision_level = rev;
    pcids.code_type = codetype;
    SetLastImageInRom( last );
}


istream& PciDataStructure::Read( istream& is )
{
    is.read( pcids.signature, sizeof(pcids.signature) );
    ReadUShort( is, pcids.vendor );
    ReadUShort( is, pcids.device );
    ReadUShort( is, pcids.vpd_offset );
    ReadUShort( is, pcids.struct_length );
    ReadUChar( is, pcids.struct_revision );
    ReadUCharArray( is, pcids.class_code, sizeof(pcids.class_code) );
    ReadUShort( is, pcids.image_length );
    ReadUShort( is, pcids.revision_level );
    ReadUChar( is, pcids.code_type );
    ReadUChar( is, pcids.indicator );
    ReadUShort( is, pcids.reserved );
    return is;
}


bool PciDataStructure::IsValid() const
{
    if ( strncmp( pcids.signature, PCIDS_SIGNATURE, sizeof(pcids.signature) ) != 0 )
	return false;
    if ( pcids.struct_length != PCIDS_LENGTH )
	return false;
    if ( pcids.struct_revision != PCIDS_REVISION )
	return false;
    return true;
}


void PciDataStructure::MakeValid()
{
    memcpy( &pcids.signature, PCIDS_SIGNATURE, sizeof(pcids.signature) );
    pcids.struct_length = PCIDS_LENGTH;
    pcids.struct_revision = PCIDS_REVISION;
}
