// PciDS.h $Id: e223e312e902f57dd6ee2a657ece1be05e7013c1 $


#include <istream>
#include <cstring>

#include "rom.h"


using std::istream;


class PciDataStructure
{
 private:
    pci_data_struct_t pcids;

 public:
    PciDataStructure( bool valid = false ) : pcids() { if ( valid ) MakeValid(); }
    PciDataStructure( unsigned short venid, unsigned short devid, unsigned int imglen, unsigned short rev, unsigned char codetype, unsigned char cls[ 3 ], bool last = true );
    istream& Read( istream& is );
    unsigned char *Write( unsigned char *dest ) const { return static_cast<unsigned char *>(memcpy( dest, &pcids, sizeof(pcids) )); }
    bool IsValid() const;
    void MakeValid();
    unsigned short GetVendorID() const { return pcids.vendor; }
    unsigned short GetDeviceID() const { return pcids.device; }
    void SetVendorAndDeviceIDs( unsigned short v, unsigned short d ) { pcids.vendor = v; pcids.device = d; }
    void SetVPDOffset( unsigned short offset ) { pcids.vpd_offset = offset; }
    unsigned short GetVPDOffset() const { return pcids.vpd_offset; }
    unsigned int GetDataStructureLength() const { return pcids.struct_length; }
    unsigned int GetDataStructureRevision() const { return pcids.struct_revision; }
    unsigned char GetClass() const { return pcids.class_code[ 2 ]; }
    unsigned char GetSubClass() const { return pcids.class_code[ 1 ]; }
    unsigned char GetProgIF() const { return pcids.class_code[ 0 ]; }
    void SetClassCode( unsigned char cls[ 3 ] ) { pcids.class_code[ 0 ] = cls[ 0 ]; pcids.class_code[ 1 ] = cls[ 1 ]; pcids.class_code[ 2 ] = cls[ 2 ]; }
    void SetClassCode( unsigned char cls, unsigned char subcls = 0, unsigned char progif = 0 ) { pcids.class_code[ 2 ] = cls; pcids.class_code[ 1 ] = subcls; pcids.class_code[ 0 ] = progif; }
    unsigned int GetImageLength() const { return pcids.image_length * PCIDS_IMAGE_LENGTH_MULTIPLE; }
    void SetImageLength( unsigned int imglen ) { pcids.image_length = (imglen + PCIDS_IMAGE_LENGTH_MULTIPLE - 1) / PCIDS_IMAGE_LENGTH_MULTIPLE; }
    unsigned short GetRevisionLevel() const { return pcids.revision_level; }
    void SetRevisionLevel( unsigned short rev ) { pcids.revision_level = rev; }
    unsigned char GetCodeType() const { return pcids.code_type; }
    void SetCodeType( unsigned char t ) { pcids.code_type = t; }
    bool IsLastImageInRom() const { return pcids.indicator & PCIDS_IND_LAST_IMAGE; }
    void SetLastImageInRom( bool last ) { pcids.indicator = last ? PCIDS_IND_LAST_IMAGE : 0; }
};

inline istream& operator>>( istream& is, PciDataStructure& pds ) { return pds.Read( is ); }
