/* s5920bootimg.c $Id: a1bfe44a68994d8bd610184dc6baa56ef097053e $

Included in mks5920bootimg.c to define the contents of the S5920 boot image.

*/


/* Bring in pciaerhw for the definitions of the sizes of the memory mapped
   regions allocated to Xilinx registers, FIFOs and the mapper SRAM. */

#include "../../driver/pciaerhw.h"

/* The above kernel include uses u32 as a name for a uint_32.
   The following define allows that to work here too. */

#define u32 uint32_t

/* Definitions */

bootimg.vendor_id = PCI_VENDOR_ID_AMCC;
bootimg.device_id = PCI_DEVICE_ID_AMCC_S5920;

/* Special modes - refer to p2-129 of AMCC data sheet */

bootimg.special_modes = S5920_SPECIAL_MODES_TARGET_LATENCY_ENB
                      | S5920_SPECIAL_MODES_RETRY_FLUSH_ENB
                      | S5920_SPECIAL_MODES_WRITE_FIFO_MODE;

bootimg.revision_id = 0;	/* AMCC recommends against changing this value */

base_class(bootimg.class_code) = PCI_BASE_CLASS_COMMUNICATION;
sub_class(bootimg.class_code) = PCI_SUB_CLASS(PCI_CLASS_COMMUNICATION_OTHER);
prog_if(bootimg.class_code) = 0;

bootimg.header_type = PCI_HEADER_TYPE_NORMAL;	/* Other values are for bridges */

bootimg.self_test = S5920_NVRAM_SELF_TEST_NOT_DESIRED;	/* Self-test not supported on PCI-AER */

bootimg.reqd_lsb_base_addr_0 = S5920_LSB_BADR0_MEM;

bootimg.reqd_valid_1 = S5920_NVRAM_REQD_VALID_1;
bootimg.reqd_valid_2 = S5920_NVRAM_REQD_VALID_2;
bootimg.reqd_valid_3 = S5920_NVRAM_REQD_VALID_3;

bootimg.base_addr_1 = S5920_BADR_PASSTHRU_REGION_WIDTH_32	/* Xilinx regs */
                    | PCI_BASE_ADDRESS_SPACE_MEMORY
                    | PCI_BASE_ADDRESS_MEM_TYPE_32
                    | (PCI_BASE_ADDRESS_MEM_MASK & ~((u32)XILINXREGS_SIZE - 1));

bootimg.base_addr_2 = S5920_BADR_PASSTHRU_REGION_WIDTH_32	/* FIFOs */
                    | PCI_BASE_ADDRESS_SPACE_MEMORY
                    | PCI_BASE_ADDRESS_MEM_TYPE_32
                    | (PCI_BASE_ADDRESS_MEM_MASK & ~(IDT7205_DEPTH * (u32)sizeof(u32) - 1));

bootimg.base_addr_3 = S5920_BADR_PASSTHRU_REGION_WIDTH_32	/* Mapper SRAM */
                    | PCI_BASE_ADDRESS_SPACE_MEMORY
                    | PCI_BASE_ADDRESS_MEM_TYPE_32
                    | (PCI_BASE_ADDRESS_MEM_MASK & ~((u32)MAPPERSRAM_SIZE - 1));

bootimg.base_addr_4 = S5920_BADR_PASSTHRU_REGION_DISABLED; /* Nothing */

/* We could put our own SVID here if we applied to the PCISIG for one, in order
   to distinguish between the PCI-AER board and other boards using the AMCC S5920. */

bootimg.svid = 0;
bootimg.sid = 0;

bootimg.exp_rom_base_addr = 0;	/* No ROM access */

bootimg.int_line = PCI_INTERRUPT_LINE_UNK; /* IRQ is unknown at boot time */
bootimg.int_pin = PCI_INTERRUPT_PIN_INTA;  /* PCI-AER is hardwired to use INTA */
