// VpdDescriptor.cc $Id: 10fbd83af1c7abc029342fcfd00d66332fd95205 $


#include <climits>
#include <cstring>

#include "VpdDescriptor.h"
#include "ReadUtil.h"


static inline unsigned char LoByte( unsigned short h ) { return h & 0xFF; }
static inline unsigned char HiByte( unsigned short h ) { return (h >> 8) & 0xFF; }


unsigned char VpdDescriptor::ByteSum() const
{
    unsigned char ByteSum = Tag + LoByte( Length ) + HiByte( Length );
    for ( unsigned int iByte = 0; iByte < Length; ++iByte )
	ByteSum += pContent[ iByte ];
    return ByteSum;
}


ostream& VpdDescriptor::BinaryWrite( ostream& os ) const
{
    os << static_cast<unsigned char>(Tag) << LoByte( Length ) << HiByte( Length );
    os.write( reinterpret_cast<const char *>(pContent), Length );
    return os;
}


istream& VpdDescriptor::BinaryRead( istream& is )
{
    delete [] pContent;
    Length = 0;

    is.ignore();
    ReadUShort( is, Length );
    pContent = new unsigned char [ Length ];
    ReadUCharArray( is, pContent, Length );

    return is;
}


bool VpdIdStringDescriptor::Set( const string& s )
{
    if ( s.empty() || (s.length() > USHRT_MAX) )
	return false;

    delete [] pContent;

    Length = s.length();
    pContent = new unsigned char [ Length ];
    strncpy( (char *)pContent, s.c_str(), Length );
    return true;
}


bool VpdReadOnlyDescriptor21::SetKeyword( const string& Keyword, const string& Data )
{
    KeywordMap[ Keyword ] = Data;
    if ( BinaryStructExists() )
	DeleteBinaryStruct();
    return true;
}


// string VpdReadOnlyDescriptor21::GetKeyword( const string& Keyword ) const
// {
//     map<string,string>::const_iterator pos;
//     pos = KeywordMap.find( Keyword );
//     return (pos != KeywordMap.end()) ? pos->second : string();
// }


void VpdReadOnlyDescriptor21::ForEachKeyword( void f( const string&, const string& ) ) const
{
    map<string,string>::const_iterator pos;
    for ( pos = KeywordMap.begin(); pos != KeywordMap.end(); ++pos )
	f( pos->first, pos->second );
}


unsigned char VpdReadOnlyDescriptor21::ByteSum() const
{
    if ( !BinaryStructExists() )
	const_cast<VpdReadOnlyDescriptor21*>(this)->CreateBinaryStructFromMap();
    return VpdDescriptor::ByteSum();
}


ostream& VpdReadOnlyDescriptor21::BinaryWrite( ostream& os ) const
{
    if ( !BinaryStructExists() )
	const_cast<VpdReadOnlyDescriptor21*>(this)->CreateBinaryStructFromMap();
    return VpdDescriptor::BinaryWrite( os );
}


istream& VpdReadOnlyDescriptor21::BinaryRead( istream& is )
{
    if ( BinaryStructExists() )
	DeleteBinaryStruct();
    VpdDescriptor::BinaryRead( is );
    KeywordMap.clear();
    FillMapFromBinaryStruct();
    return is;
}


void VpdReadOnlyDescriptor21::CreateBinaryStructFromMap()
{
    if ( pContent != 0 )
	return;

    Length = 0;

    map<string,string>::const_iterator pos;

    for ( pos = KeywordMap.begin(); pos != KeywordMap.end(); ++pos )
	Length += sizeof(vpd_keyword_t) - 1 + pos->second.length();
    
    pContent = new unsigned char [ Length ];

    unsigned char *pNextKeyword = pContent;
    vpd_keyword_t *pKeyword;
    for ( pos = KeywordMap.begin(); pos != KeywordMap.end(); ++pos )
	{
	    pKeyword = reinterpret_cast<vpd_keyword_t *>(pNextKeyword);
	    pKeyword->keyword[ 0 ] = pos->first[ 0 ];
	    pKeyword->keyword[ 1 ] = pos->first[ 1 ];
	    pKeyword->length = pos->second.length();
	    strncpy( pKeyword->data, pos->second.c_str(), pKeyword->length );
	    pNextKeyword += sizeof(vpd_keyword_t) - 1 + pKeyword->length;
	}
}


void VpdReadOnlyDescriptor21::FillMapFromBinaryStruct()
{
    vpd_keyword_t *pKeyword;
    for ( unsigned char *pNextKeyword = pContent; pNextKeyword < pContent + Length; pNextKeyword += sizeof(vpd_keyword_t) - 1 + pKeyword->length )
	{
	    pKeyword = reinterpret_cast<vpd_keyword_t *>(pNextKeyword);
	    string Keyword( pKeyword->keyword, 2 );
	    string Data( pKeyword->data, pKeyword->length );
	    KeywordMap[ Keyword ] = Data;
	}
}
