// VpdDescriptor.h $Id: 55ec6d22478c723f04493c0be4b189b36cf3afeb $

#include <algorithm>
#include <map>
#include <iostream>
#include <string>

#include "vpd.h"

using std::map;
using std::istream;
using std::ostream;
using std::string;

class VpdDescriptor
{
 public:
    typedef enum {
	IDStringTag = VPD_ID_STRING_DESCRIPTOR_TAG,
	ReadOnlyTag = VPD_READ_ONLY_DESCRIPTOR_TAG,
	EndTag21 = VPD_END_DESCRIPTOR_TAG_21
    } e_tag_type;

 protected:
    e_tag_type Tag;
    unsigned short Length;
    unsigned char *pContent;

    VpdDescriptor( e_tag_type Type ) : Tag( Type ), Length( 0 ), pContent( 0 ) {}
    virtual ~VpdDescriptor() { delete [] pContent; }

 public:
    virtual unsigned char ByteSum() const;
    virtual ostream& BinaryWrite( ostream& os ) const;
    virtual istream& BinaryRead( istream& is );

 private:
    VpdDescriptor( const VpdDescriptor& dtor );
    const VpdDescriptor& operator=( const VpdDescriptor& dtor );
};

inline ostream& operator<<( ostream& os, const VpdDescriptor& dtor ) { return dtor.BinaryWrite( os ); }
inline istream& operator>>( istream& is, VpdDescriptor& dtor ) { return dtor.BinaryRead( is ); }

class VpdIdStringDescriptor : public VpdDescriptor
{
 public:
    VpdIdStringDescriptor() : VpdDescriptor( IDStringTag ) {}
    bool Set( const string& s );
    string Get() const { return (pContent == 0) ? string() : string( (char*)pContent, Length ); }
    bool IsDefined() const { return pContent != 0; }
};

class VpdReadOnlyDescriptor21 : public VpdDescriptor
{
 private:
    std::map<string,string> KeywordMap;

 public:
    VpdReadOnlyDescriptor21() : VpdDescriptor( ReadOnlyTag ) {}
    virtual unsigned char ByteSum() const;
    virtual ostream& BinaryWrite( ostream& os ) const;
    virtual istream& BinaryRead( istream& is );
    bool SetKeyword( const string& Keyword, const string& Data );
    //    string GetKeyword( const string& Keyword ) const;
    void ForEachKeyword( void f( const string&, const string& ) ) const;

 private:
    bool BinaryStructExists() const { return pContent != 0; }
    void CreateBinaryStructFromMap();
    void DeleteBinaryStruct() { delete [] pContent; Length = 0; pContent = 0; }
    void FillMapFromBinaryStruct();
};

class VpdEndTagDescriptor21 : public VpdDescriptor
{
 public:
    VpdEndTagDescriptor21() : VpdDescriptor( EndTag21 ) {}
    virtual unsigned char ByteSum() const { return Tag; }
    virtual ostream& BinaryWrite( ostream& os ) const { return os << static_cast<unsigned char>(Tag); }
    virtual istream& BinaryRead( istream& is ) { return is.ignore(); } // @@@ check character read is correct
};

