/*
Code common to multiple test progs

common.cc $Id: 0fa5857534a01b632f55dd698153ac8d0135f2a5 $
*/

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/types.h>

#include "pciaer.h"
#include "common.h"


static bool signalled = false;

void signal_handler( int signum )
{
  signalled = true;
}

bool is_signalled()
{
    return signalled;
}

#ifdef __cplusplus
#include "../s5920nvram-support/cmdline.h"
int GetNumOpt( const CCmdLine& cl, char opt, int default_result )
{
    string s;
    int result;
    if ( cl.Opt( opt, s ) )
	result = (int)strtol( s.c_str(), NULL, 0 );
    else
	result = default_result;
    return result;
}
#endif

void print_version( int handle )
{
  pciaer_version_info_t v;

  if ( ioctl( handle, PCIAER_IOC_GET_VERSION_INFO, &v ) != 0 )
    {
      perror( "PCIAER_IOC_GET_VERSION_INFO" );
      exit( errno );
    }
  printf( "Version info: {0x%04hX;0x%04hX;0x%04hX;0x%02X}\n",
	  v.driver_version,
	  v.fpga1_release,
	  v.fpga2_release,
	  (int)v.s5920_revision_id
	);
}


void test_lseek( int handle )
{
  if ( lseek( handle, sizeof(unsigned int), SEEK_CUR ) != -1 || errno != ESPIPE )
      {
	  fprintf( stderr, "lseek failed to fail\n" );
	  exit( ESPIPE );
      }
}


unsigned int test_fifo_depth( int handle )
{
  unsigned int fifo_depth;
  struct stat stat_buffer;

  if ( fstat( handle, &stat_buffer ) != 0 )
      {
	  perror( "fstat" );
	  exit( errno );
      }

  fifo_depth = stat_buffer.st_blksize * 2 / sizeof(unsigned int);

  printf( "Fifo depth = %d\n", fifo_depth );

  return fifo_depth;
}
