/*
Dumps and decodes PCI-AER FPGA registers.

fpgaregs.c $Id: f7896b8d58fbecc2708d100993be5dd1aa245ab2 $
*/


#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/param.h>

#include "pciaer.h"
#include "pciaerhw.h"

#define XILINX_PATH 	"/sys/kernel/debug/pciaer/pciaer0/xilinx/"

#define ReadRegister(R)	ReadValueFromFile(XILINX_PATH #R)


static const char Id[] = "$Id: f7896b8d58fbecc2708d100993be5dd1aa245ab2 $";


char *utob( unsigned int u, char *pDst )
{
    int i;
    for ( i = 31; i >= 0; i-- )
	pDst[ 31 - i ] = (u & (1 << i)) ? '1' : '0';
    pDst[ 32 ] = '\0';
    return pDst;
}


char *MaskString( char *pSrc, char *pDst, unsigned int uMask, char chMask )
{
    int i;
    for ( i = 31; i >= 0; i-- )
	pDst[ 31 - i ] = (uMask & (1 << i)) ? pSrc[ 31 - i ] : chMask;
    pDst[ 32 ] = '\0';
    return pDst;
}


unsigned int ReadValueFromFile( char *pchPath )
{
	FILE *file;
	char HexBuffer[ 12 ];

    file = fopen( pchPath, "r" );
    if ( file == NULL )
    {
    	perror( "fopen" );
    	exit( errno );
    }

    fgets( HexBuffer, sizeof(HexBuffer), file );

    fclose( file );

    return (unsigned int)strtoul( HexBuffer, NULL, 16 );
}


void StatusRegA1( void )
{
    unsigned int uSRA1;
    char szSRA1[ 33 ];
    char szMasked[ 33 ];

    uSRA1 = ReadRegister( sra1 );
    utob( uSRA1, szSRA1 );
    printf( "SRA1 0x%08X      %s\n", uSRA1, szSRA1 );
    printf( "SRA1 used            %s\n", MaskString( szSRA1, szMasked, 0x1FFF, 'x' ) );
    printf( "SRA1 MapStsFFInt     %s\n", MaskString( szSRA1, szMasked, SRA1_INT_FF_MAP, '.' ) );
    printf( "SRA1 SeqStsFHInt     %s\n", MaskString( szSRA1, szMasked, SRA1_INT_FH_SEQ, '.' ) );
    printf( "SRA1 SeqStsFEInt     %s\n", MaskString( szSRA1, szMasked, SRA1_INT_FE_SEQ, '.' ) );
    printf( "SRA1 MonStsFFInt     %s\n", MaskString( szSRA1, szMasked, SRA1_INT_FF_MON, '.' ) );
    printf( "SRA1 MonStsFHInt     %s\n", MaskString( szSRA1, szMasked, SRA1_INT_FH_MON, '.' ) );
    printf( "SRA1 MapF?           %s\n", MaskString( szSRA1, szMasked, MAP_FIFO_FLAGS_MASK, '.' ) );
    printf( "SRA1 SeqF?           %s\n", MaskString( szSRA1, szMasked, SEQ_FIFO_FLAGS_MASK, '.' ) );
    printf( "SRA1 MonF?           %s\n\n", MaskString( szSRA1, szMasked, MON_FIFO_FLAGS_MASK, '.' ) );
}


void ConfigRegA1( void )
{
    unsigned int uCRA1;
    char szCRA1[ 33 ];
    char szMasked[ 33 ];

    uCRA1 = ReadRegister( cra1 );
    utob( uCRA1, szCRA1 );
    printf( "CRA1 0x%08X      %s\n", uCRA1, szCRA1 );
    printf( "CRA1 used            %s\n", MaskString( szCRA1, szMasked, 0x7FFF, 'x' ) );
    printf( "CRA1 MapFMR          %s\n", MaskString( szCRA1, szMasked, CRA1_MAP_IN_FIFO_MR_, '.' ) );
    printf( "CRA1 EnMapIN         %s\n", MaskString( szCRA1, szMasked, CRA1_MAP_IN_DISABLE, '.' ) );
    printf( "CRA1 SeqFRT          %s\n", MaskString( szCRA1, szMasked, CRA1_SEQ_FIFO_RT_, '.' ) );
    printf( "CRA1 SeqFMR          %s\n", MaskString( szCRA1, szMasked, CRA1_SEQ_FIFO_MR_, '.' ) );
    printf( "CRA1 SeqChSel        %s\n", MaskString( szCRA1, szMasked, CRA1_SEQ_CH_SEL_MASK, '.' ) );
    printf( "CRA1 EnSeq           %s\n", MaskString( szCRA1, szMasked, CRA1_SEQ_DISABLE, '.' ) );
    printf( "CRA1 MonFRT          %s\n", MaskString( szCRA1, szMasked, CRA1_MON_FIFO_RT_, '.' ) );
    printf( "CRA1 MonFMR          %s\n", MaskString( szCRA1, szMasked, CRA1_MON_FIFO_MR_, '.' ) );
    printf( "CRA1 EN_TIMELbl      %s\n", MaskString( szCRA1, szMasked, CRA1_DIS_MON_TIME_LBL, '.' ) );
    printf( "CRA1 En_Mon_Ch       %s\n", MaskString( szCRA1, szMasked, CRA1_MON_IGN_CH_MASK, '.' ) );
    printf( "CRA1 EnMon           %s\n\n", MaskString( szCRA1, szMasked, CRA1_MON_DISABLE, '.' ) );
}


void ConfigRegB1( void )
{
    unsigned int uCRB1;
    char szCRB1[ 33 ];
    char szMasked[ 33 ];

    uCRB1 = ReadRegister( crb1 );
    utob( uCRB1, szCRB1 );
    printf( "CRB1 0x%08X      %s\n", uCRB1, szCRB1 );
    printf( "CRB1 used            %s\n", MaskString( szCRB1, szMasked, 0x3FF, 'x' ) );
    printf( "CRB1 EnMapCh         %s\n", MaskString( szCRB1, szMasked, CRB1_MAP_IGN_CH_MASK, '.' ) );
    printf( "CRB1 ArbCfg          %s\n", MaskString( szCRB1, szMasked, CRB1_ARB_CFG_MASK, '.' ) );
    printf( "CRB1 AERClkCfg       %s\n", MaskString( szCRB1, szMasked, CRB1_AER_CLOCK_MASK, '.' ) );
    printf( "CRB1 RstTIME         %s\n", MaskString( szCRB1, szMasked, CRB1_TIME_RESET_, '.' ) );
    printf( "CRB1 EnTIME          %s\n\n", MaskString( szCRB1, szMasked, CRB1_TIME_DISABLE, '.' ) );
}


void ConfigRegC1( void )
{
    unsigned int uCRC1;
    char szCRC1[ 33 ];
    char szMasked[ 33 ];

    uCRC1 = ReadRegister( crc1 );
    utob( uCRC1, szCRC1 );
    printf( "CRC1 0x%08X      %s\n", uCRC1, szCRC1 );
    printf( "CRC1 used            %s\n", MaskString( szCRC1, szMasked, 0x601F, 'x' ) );
    printf( "CRC1 SMDebug         %s\n", MaskString( szCRC1, szMasked, CRC1_DIS_SM_DEBUG, '.' ) );
    printf( "CRC1 EnAntiMeta      %s\n", MaskString( szCRC1, szMasked, CRC1_EN_ANTI_META, '.' ) );
    printf( "CRC1 EnInt_FF_MAP    %s\n", MaskString( szCRC1, szMasked, CRC1_DIS_INT_FF_MAP, '.' ) );
    printf( "CRC1 EnInt_HF_SEQ    %s\n", MaskString( szCRC1, szMasked, CRC1_DIS_INT_FH_SEQ, '.' ) );
    printf( "CRC1 EnInt_EF_SEQ    %s\n", MaskString( szCRC1, szMasked, CRC1_DIS_INT_FE_SEQ, '.' ) );
    printf( "CRC1 EnInt_FF_MON    %s\n", MaskString( szCRC1, szMasked, CRC1_DIS_INT_FF_MON, '.' ) );
    printf( "CRC1 EnInt_HF_MON    %s\n\n", MaskString( szCRC1, szMasked, CRC1_DIS_INT_FH_MON, '.' ) );
}


void StatusRegA2( void )
{
    unsigned int uSRA2;
    char szSRA2[ 33 ];
    char szMasked[ 33 ];

    uSRA2 = ReadRegister( sra2 );
    utob( uSRA2, szSRA2 );
    printf( "SRA2 0x%08X      %s\n", uSRA2, szSRA2 );
    printf( "SRA2 used            %s\n", MaskString( szSRA2, szMasked, 1, 'x' ) );
    printf( "SRA2 MMU_Busy        %s\n\n", MaskString( szSRA2, szMasked, SRA2_MMU_IDLE, '.' ) );
}


void ConfigRegA2( void )
{
    unsigned int uCRA2;
    char szCRA2[ 33 ];
    char szMasked[ 33 ];

    uCRA2 = ReadRegister( cra2 );
    utob( uCRA2, szCRA2 );
    printf( "CRA2 0x%08X      %s\n", uCRA2, szCRA2 );
    printf( "CRA2 used            %s\n", MaskString( szCRA2, szMasked, 0x3F, 'x' ) );
    printf( "CRA2 KdAwAER         %s\n", MaskString( szCRA2, szMasked, CRA2_AER_PROTOCOL_MASK, '.' ) );
    printf( "CRA2 MapperOutEN     %s\n", MaskString( szCRA2, szMasked, CRA2_MAP_OUT_DISABLE, '.' ) );
    printf( "CRA2 AEROutCfg       %s\n", MaskString( szCRA2, szMasked, CRA2_AER_OUT_CFG_MASK, '.' ) );
    printf( "CRA2 CfgMapperOut    %s\n\n", MaskString( szCRA2, szMasked, CRA2_MAP_OUT_CFG_MASK, '.' ) );
}


void PrintDriverVersion( void )
{
	int handle;
    pciaer_version_info_t v;

    handle = open( "/dev/aernvr0", O_RDONLY );
    if ( handle == -1 )
    {
    	perror( "open" );
    	exit ( 1 );
    }

    if ( ioctl( handle, PCIAER_IOC_GET_VERSION_INFO, &v ) != 0 )
	{
	    perror( "PCIAER_IOC_GET_VERSION_INFO" );
	    exit( 1 );
	}

    printf( "Driver version info: {0x%04hX;0x%04hX;0x%04hX;0x%02X}\n\n",
	    v.driver_version,
	    v.fpga1_release,
	    v.fpga2_release,
	    (int)v.s5920_revision_id
	  );

    close( handle );
}


int main( int argc, char *argv[] )
{
  char HostName[ MAXHOSTNAMELEN + 1 ];

  // Report own version no. and machine on which we're running

  gethostname( HostName, MAXHOSTNAMELEN + 1 );
  printf( "fpgaregs %s running on %s\n\n", Id, HostName );

  PrintDriverVersion();

  StatusRegA1();

  ConfigRegA1();

  ConfigRegB1();

  ConfigRegC1();

  StatusRegA2();

  ConfigRegA2();

  return 0;
}
