/*
Test code for PCI-AER board 'Stuck MMU Busy' bug.

mmubusytest.c $Id: 5c54be9da61388f79f65d502602e0a4232c51395 $
*/


#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/mman.h>

#include "pciaer.h"


#define MAX_AE	((1 << 14) - 1)


unsigned short ae_list[] = { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10 };

int main( int argc, char *argv[] )
{
  int handle;
  pciaer_version_info_t v;
  int seq_channel;
  int config;
  int channels;
  int fifo_flags;
  int state;
  int protocol;
  pciaer_mapping_t mapping;
  unsigned short ae;
  int d;

  handle = open( "/dev/aermap0", O_RDWR );
  if ( handle == -1 )
    {
      perror( "open" );
      exit( 1 );
    }

  if ( ioctl( handle, PCIAER_IOC_GET_VERSION_INFO, &v ) != 0 )
    {
      perror( "PCIAER_IOC_GET_VERSION_INFO" );
      exit( 1 );
    }
  printf( "Version info: {0x%04hX;0x%04hX;0x%04hX;0x%02X}\n",
	  v.driver_version,
	  v.fpga1_release,
	  v.fpga2_release,
	  (int)v.s5920_revision_id
	);

  if ( ioctl( handle, PCIAER_IOC_GET_SEQ_ARB_CHANNEL, &seq_channel ) != 0 )
    {
      perror( "PCIAER_IOC_GET_SEQ_ARB_CHANNEL" );
      exit( 1 );
    }
  printf( "Sequencer arbiter channel %d\n", seq_channel );

  if ( ioctl( handle, PCIAER_IOC_GET_ARB_CONFIG, &config ) != 0 )
    {
      perror( "PCIAER_IOC_GET_ARB_CONFIG" );
      exit( 1 );
    }
  printf( "Arbiter config %d\n", config );

  if ( ioctl( handle, PCIAER_IOC_GET_MAP_CH_SEL, &channels ) != 0 )
    {
      perror( "PCIAER_IOC_SET_MAP_CH_SEL" );
      exit( 1 );
    }
  printf( "Channel selection bitmask 0x%08X\n", channels );

  if ( ioctl( handle, PCIAER_IOC_GET_MAP_FIFO_FLAGS, &fifo_flags ) != 0 )
    {
      perror( "PCIAER_IOC_GET_MAP_FIFO_FLAGS" );
      exit( 1 );
    }
  printf( "Fifo flags 0x%08X\n", fifo_flags );

  if ( ioctl( handle, PCIAER_IOC_GET_MAP_OUT_STATE, &state ) != 0 )
    {
      perror( "PCIAER_IOC_GET_MAP_OUT_STATE" );
      exit( 1 );
    }
  printf( "Mapper out state 0x%08X\n", state );

  if ( ioctl( handle, PCIAER_IOC_GET_MAP_OUT_CONFIG, &config ) != 0 )
    {
      perror( "PCIAER_IOC_GET_MAP_OUT_CONFIG" );
      exit( 1 );
    }
  printf( "Mapper out config %d\n", config );

  if ( ioctl( handle, PCIAER_IOC_GET_MAP_DEMUX_CONFIG, &config ) != 0 )
    {
      perror( "PCIAER_IOC_GET_MAP_DEMUX_CONFIG" );
      exit( 1 );
    }
  printf( "Demux config %d\n", config );

  if ( ioctl( handle, PCIAER_IOC_GET_MAP_AER_PROTOCOL, &protocol ) != 0 )
    {
      perror( "PCIAER_IOC_GET_MAP_AER_PROTOCOL" );
      exit( 1 );
    }
  printf( "Protocol %d\n", protocol );

  printf( "Disabling Mapper out\n" );
  if ( ioctl( handle, PCIAER_IOC_DISABLE_MAP_OUT, 1 ) != 0 )
    {
      perror( "PCIAER_IOC_DISABLE_MAP_OUT" );
      exit( 1 );
    }

  if ( ioctl( handle, PCIAER_IOC_GET_MAP_OUT_STATE, &state ) != 0 )
    {
      perror( "PCIAER_IOC_GET_MAP_OUT_STATE" );
      exit( 1 );
    }
  printf( "Mapper out state now 0x%08X\n", state );
  
  printf( "Enabling Mapper out\n" );
  if ( ioctl( handle, PCIAER_IOC_ENABLE_MAP_OUT, 1 ) != 0 )
    {
      perror( "PCIAER_IOC_ENABLE_MAP_OUT" );
      exit( 1 );
    }

  if ( ioctl( handle, PCIAER_IOC_GET_MAP_OUT_STATE, &state ) != 0 )
    {
      perror( "PCIAER_IOC_GET_MAP_OUT_STATE" );
      exit( 1 );
    }
  printf( "Mapper out state now 0x%08X\n", state );
  
  printf( "Clearing Mapper out\n" );
  if ( ioctl( handle, PCIAER_IOC_MAP_CLEAR ) != 0 )
    {
      perror( "PCIAER_IOC_MAP_CLEAR" );
      exit( 1 );
    }

  if ( ioctl( handle, PCIAER_IOC_GET_MAP_OUT_STATE, &state ) != 0 )
    {
      perror( "PCIAER_IOC_GET_MAP_OUT_STATE" );
      exit( 1 );
    }
  printf( "Mapper out state now 0x%08X\n", state );

  for ( ae = 0; ae <= MAX_AE; ae++ )
    {
      printf( "AE %hu\n", ae );
      for ( d = 0; d < 10; d++ )
	ae_list[ d ] = ae;

      mapping.source_ae = ae;
      mapping.dest_count = 10; 
      mapping.dest_ae = ae_list;
      if ( ioctl( handle, PCIAER_IOC_SET_MAP_MAPPING, &mapping ) != 0 )
	{
	  perror( "PCIAER_IOC_SET_MAP_MAPPING" );
	  exit( 1 );
	}
    }

  while ( 1 )
    {
      printf( "Disabling Mapper out\n" );
      if ( ioctl( handle, PCIAER_IOC_DISABLE_MAP_OUT, 1 ) != 0 )
	{
	  perror( "PCIAER_IOC_DISABLE_MAP_OUT" );
	  exit( 1 );
	}

      if ( ioctl( handle, PCIAER_IOC_GET_MAP_OUT_STATE, &state ) != 0 )
	{
	  perror( "PCIAER_IOC_GET_MAP_OUT_STATE" );
	  exit( 1 );
	}
      printf( "Mapper out state now 0x%08X\n", state );
  
      printf( "Enabling Mapper out\n" );
      if ( ioctl( handle, PCIAER_IOC_ENABLE_MAP_OUT, 1 ) != 0 )
	{
	  perror( "PCIAER_IOC_ENABLE_MAP_OUT" );
	  exit( 1 );
	}
      
      if ( ioctl( handle, PCIAER_IOC_GET_MAP_OUT_STATE, &state ) != 0 )
	{
	  perror( "PCIAER_IOC_GET_MAP_OUT_STATE" );
	  exit( 1 );
	}
      printf( "Mapper out state now 0x%08X\n", state );
    }

  close( handle );
    
  return 0;
}
