// pciaerstats.cc: A program to read and display statistics from the PCI-AER driver, pciaer.o

// pciaerstats.cc $Id: 6d16f0345dc7926fde6ec28e28b481e24644a880 $

// Supports command line switches:
//   -o  reset Only
//   -r  refresh and Reset
//   -t  get Timestamp

#include <iostream>
#include <iomanip>

#include <cerrno>
#include <cstdio>
#include <cstdlib>

#include <fcntl.h>

#include "pciaer.h"
#include "../s5920nvram-support/cmdline.h"
#include "PciaerStats.h"

int main( int argc, char *argv[] )
{
  CCmdLine command_line( argc, argv, CCmdLine::flagAllowArgOptGroup | CCmdLine::flagAllowOptArgNoSpace );
  int h;
  int e;

  if ( command_line.Parse( "ort" ) != NULL )
      exit( EINVAL );

  if ( command_line.GetNonOptArgCount() != 1 )
      {
	  std::cerr << "Supply device node to open!\n";
	  exit( 1 );
      }

  h = open( command_line.Arg( 1 ), O_RDONLY );
  if ( h == -1 )
      {
	  perror( "open" );
	  exit( 2 );
      }

  PciaerStats s( h );

  if ( command_line.Opt( 'o' ) )
      e = s.Reset();
  else if ( command_line.Opt( 'r' ) )
      e = s.RefreshAndReset();
  else
      e = s.Refresh();

  if ( e != 0 )
      {
	  perror( "ioctl" );
	  exit( 3 );
      }

  if ( !command_line.Opt( 'o' ) )
      {
	  std::cout << command_line.Arg( 1 ) << ":\n" << s;
	  if ( command_line.Opt( 't' ) )
	      {
 		  const struct timeval & tv( s.GetTimestamp() );
 		  std::cout << std::setw( 11 ) << tv.tv_sec << '.' << std::setw( 6 ) << std::setfill( '0' ) << tv.tv_usec << " s (timestamp)\n";
	      }
      }

  close( h );
  return 0;
}
