/*
Code to test the PCIAER_IOC_GET_PCI_INFO ioctl in the PCI-AER driver, pciaer.o

pciinfotest.c $Id: 4b85e1e4e0ad17687c2848f53078324899121da7 $
*/

#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
//#include <sys/ioctl.h>

#include "pciaer.h"
#include "pciaerlib.h"

#define MAX_SLOT_NAME_LEN	20
#define MAX_DEV_NAME_LEN	96


int main( int argc, char *argv[] )
{
  int use_lib;
  char *dev_path;
  int h;
  pciaer_pci_info_t p;
  int e;
  char SlotName[ MAX_SLOT_NAME_LEN ];
  char DeviceName[ MAX_DEV_NAME_LEN ];

  switch( argc )
  {
	case 1:
		fprintf( stderr, "Supply device node to open!\n" );
		exit( EINVAL );
	case 2:
		use_lib = 0;
		dev_path = argv[ 1 ];
		break;
	case 3:
		if ( strcmp( argv[ 1 ], "-l" ) == 0 )
		{
			use_lib = 1;
			dev_path = argv[ 2 ];
		}
		else
		{
			fprintf( stderr, "Invalid option!\n" );
			exit( EINVAL );
		}
		break;
	default:
		errno = E2BIG;
		perror( NULL );
		exit( errno );
  }

  h = open( dev_path, O_RDONLY );
  if ( h == -1 )
      {
	  perror( "open" );
	  exit( errno );
      }

  p.slot_name = SlotName;
  p.device_name = DeviceName;

  if ( use_lib )
	e = PciaerGetPciInfo( h, &p );
  else
	e = ENOTTY;

  if ( e != 0 )
  {
	errno = e;
	perror( use_lib ? "lib" : "ioctl" );
	exit( errno );
  }

  printf( "Slot-Name \"%s\"\n"
	  "Bus %d\n"
	  "Slot %02x\n"
	  "Function %x\n"
	  "Device-Name \"%s\"\n"
	  "Vendor-ID %04X\n"
	  "Device-ID %04X\n"
	  "Subsystem-Vendor-ID %04X\n"
	  "Subsystem-Device-ID %04X\n"
	  "Base-class %Xh\n"
	  "Sub-class %02Xh\n"
	  "Prog.-I/F %02Xh\n"
	  "IRQ %d\n"
	  ,
	  p.slot_name ? p.slot_name : "NULL",
	  p.bus,
	  p.slot,
	  p.func,
	  p.device_name ? p.device_name : "NULL",
	  p.vendor,
	  p.device,
	  p.subsys_vendor,
	  p.subsys_device,
	  p.base_class,
	  p.sub_class,
	  p.prog_if,
	  p.irq
	);

  close( h );
  return 0;
}
