/*
Code to test ioctl permission checking in the PCI-AER driver, pciaer.o

permtest.c $Id: 504f67571c6992029e1f86dbdf350f526a57ca38 $
*/

#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/ioctl.h>

#include "pciaer.h"
#include "pciaerhw.h"
#include "s5920.h"

int OpenMode[ 3 ] = { O_RDONLY, O_WRONLY, O_RDWR };

void Test( char *pDescription, int handle, int nIoctl, void *pvArg, int nExpectedResult )
{
    int nRetVal;
    int e;

    fprintf( stderr, "%s\n", pDescription );
    nRetVal = ioctl( handle, nIoctl, pvArg );
    e = errno;
    if ( nExpectedResult == 0 && nRetVal == 0 )
	{
	    fprintf( stderr, "\tOK\n" );
	}
    if ( nExpectedResult == 0 && nRetVal != 0 )
	{
	    perror( "Failure - should have succeeded!" );
	    exit( 1 );
	}
    if ( nExpectedResult != 0 && nRetVal == 0 )
	{
	    perror( "Success - should have failed!" );
	    exit( 1 );
	}
    if ( nExpectedResult != 0 && nRetVal != 0 )
	{
	    fprintf( stderr,
		     "\terrno %d, expected %d, %s\n",
		       e,
		       nExpectedResult,
		       (errno == nExpectedResult) ? "OK" : "\aFAILED"
		     );
	}
}

int General()
{
  int Ioctl[ 4 ] = { PCIAER_IOC_SET_MAP_OUT_CONFIG,	// _IO   => needs -w-
		     PCIAER_IOC_GET_MAP_OUT_CONFIG,	// _IOR  => needs r--
		     PCIAER_IOC_SET_MAP_MAPPING,	// _IOW  => needs -w-
		     PCIAER_IOC_GET_MAP_MAPPING };	// _IOWR => needs r--
  int ExpectedResult[ 4 ][ 3 ] = { { EACCES, EINVAL, EINVAL },
				   { EFAULT, EACCES, EFAULT },
				   { EACCES, EFAULT, EFAULT },
				   { EFAULT, EACCES, EFAULT } };
  int h;
  int Handle;
  int i;
  char Description[ 80 ];

  for ( h = 0; h < 3; h++ )
      {
	  fprintf( stderr, "Handle %d\n", h );
	  Handle = open( "/dev/aermap0", OpenMode[ h ] );
	  if ( Handle == -1 )
	      {
		  perror( "open" );
		  return 0;
	      }
	  for ( i = 0; i < 4; i++ )
	      {
		  sprintf( Description, "Ioctl '%c', handle %d", _IOC_NR(Ioctl[ i ]), h );
		  Test( Description, Handle, Ioctl[ i ], NULL, ExpectedResult[ i ][ h ] );
	      }
	  close( Handle );
      }

  return 1;
}

typedef struct {
    int nIoctl;
    void *pvArg;
    char *pchIoctlDesc;
    int ExpectedResult[ 3 ];
} exception;

struct timeval g_tv;
pciaer_stats_t g_stats = { sizeof(pciaer_stats_t), };
int g_cFifoFills;

exception MonExceptions[] = {
    { PCIAER_IOC_RST_FIFO, NULL, "PCIAER_IOC_RST_FIFO", { 0, EACCES, 0 } },
    { PCIAER_IOC_RST_CNTR_GET_TIME, NULL, "PCIAER_IOC_RST_CNTR_GET_TIME, NULL", { EACCES, 0, 0 } },
    { PCIAER_IOC_RST_CNTR_GET_TIME, &g_tv, "PCIAER_IOC_RST_CNTR_GET_TIME, &timeval", { EACCES, EACCES, 0 } },
    { 0, NULL, NULL, { ENOTTY, ENOTTY, ENOTTY } }
};

exception SeqExceptions[] = {
    { PCIAER_IOC_RST_CNTR_GET_TIME, NULL, "PCIAER_IOC_RST_CNTR_GET_TIME, NULL", { EACCES, 0, 0 } },
    { PCIAER_IOC_RST_CNTR_GET_TIME, &g_tv, "PCIAER_IOC_RST_CNTR_GET_TIME, &timeval", { EACCES, EACCES, 0 } },
    { 0, NULL, NULL, { ENOTTY, ENOTTY, ENOTTY } }
};

exception MapExceptions[] = {
    { PCIAER_IOC_GET_MAP_MAPPED_MAP, NULL, "PCIAER_IOC_GET_MAP_MAPPED_MAP", { EFAULT, EACCES, EFAULT } },
    { 0, NULL, NULL, { ENOTTY, ENOTTY, ENOTTY } }
};

void Exception( char *pchDevAbbrev, exception *ExceptionTable )
{
    char *pModeLabel[ 3 ] = { "RO", "WO", "RW" };
    int h;
    int Handle[ 3 ];
    char DeviceName[ 20 ];
    exception *pE;
    char Description[ 80 ];

    sprintf( DeviceName, "/dev/aer%s0", pchDevAbbrev );
    
    for ( h = 0; h < 3; h++ )
	{
	    Handle[ h ] = open( DeviceName, OpenMode[ h ] );
	    if ( Handle[ h ] == -1 )
		perror( "open" );
	}

    for ( pE = ExceptionTable; pE->nIoctl != 0; pE++ )
	for ( h = 0; h < 3; h++ )
	    {
		sprintf( Description, "%s %s %s", pchDevAbbrev, pModeLabel[ h ], pE->pchIoctlDesc );
		Test( Description, Handle[ h ], pE->nIoctl, pE->pvArg, pE->ExpectedResult[ h ] );
	    }

    for ( h = 0; h < 3; h++ )
	close( Handle[ h ] );
}

int main( int argc, char *argv[] )
{
  int hMonitorRO = -1;
  pciaer_version_info_t v;
  uid_t euid;

  hMonitorRO = open( "/dev/aermon0", O_RDONLY );
  if ( hMonitorRO == -1 )
      {
	  perror( "open /dev/aermon0 O_RDONLY" );
	  exit( 1 );
      }
    
  if ( ioctl( hMonitorRO, PCIAER_IOC_GET_VERSION_INFO, &v ) != 0 )
      {
	  perror( "PCIAER_IOC_GET_VERSION_INFO" );
	  exit( 1 );
      }
  printf( "Version info: {0x%04hX;0x%04hX;0x%04hX;0x%02X}\n",
	  v.driver_version,
	  v.fpga1_release,
	  v.fpga2_release,
	  (int)v.s5920_revision_id
	);

  close( hMonitorRO );
  hMonitorRO = -1;

  euid = geteuid();

  // Test general permission checking (see comments in function for details).

  if ( !General() && euid == 0 )
      exit( 1 );

  // Test exceptions to the rule

  Exception( "mon", MonExceptions );
  Exception( "seq", SeqExceptions );
  Exception( "map", MapExceptions );

  if ( euid != 0 )
      fprintf( stderr, "Run as root for full test coverage!\n" );

  return 0;
}
