/*
Program to reset the PCI-AER board Xilinx registers via debugfs.

rstxilinx.c $Id: 4e30c8b19edf827374eb82959e72e0c131f3970d $
*/


#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "pciaer.h"


#define DEBUG_FS_PATH		"/sys/kernel/debug/pciaer/pciaer0/"
#define XILINX_RST_PATH 	DEBUG_FS_PATH "xilinx/reset"


static void WriteValueToFile( char *pPath, unsigned int uValue )
{
	FILE *file;

	file = fopen( pPath, "a" );
	if ( file == NULL )
    {
		perror( "fopen" );
		exit( errno );
    }

	fprintf( file, "%x", uValue );

	fclose( file );
}


int main( int argc, char *argv[] )
{
  int hMon;
  int hSeq;
  int hMap;
  int hNvr;

  pciaer_version_info_t v;

// Don't attempt to reset unless we can successfully open all the minor
// devices in order to make sure that no other program is using the driver
// (all three of the monitor, sequencer and mapper minor devices are 'single
// open' devices). Otherwise it is possible that the driver would get confused
// when the hardware was reset beneath it.

  hMon = open( "/dev/aermon0", O_RDONLY );
  if ( hMon == -1 )
    {
      perror( "open /dev/aermon0" );
      exit( 1 );
    }

  hSeq = open( "/dev/aerseq0", O_RDONLY );
  if ( hSeq == -1 )
    {
      perror( "open /dev/aerseq0" );
      exit( 1 );
    }

  hMap = open( "/dev/aermap0", O_RDONLY );
  if ( hMap == -1 )
    {
      perror( "open /dev/aermap0" );
      exit( 1 );
    }

  hNvr = open( "/dev/aernvr0", O_RDONLY );
  if ( hNvr == -1 )
    {
      perror( "open /dev/aernvr0" );
      exit( 1 );
    }

  // Report version info

  if ( ioctl( hNvr, PCIAER_IOC_GET_VERSION_INFO, &v ) != 0 )
    {
      perror( "PCIAER_IOC_GET_VERSION_INFO" );
      exit( 1 );
    }
  printf( "Version info: {0x%04hX;0x%04hX;0x%04hX;0x%02X}\n",
	  v.driver_version,
	  v.fpga1_release,
	  v.fpga2_release,
	  (int)v.s5920_revision_id
	);

  // Reset Xilinx chips

  WriteValueToFile( XILINX_RST_PATH, 0 );

  // Close all handles in reverse order of opening.

  close( hNvr );
  close( hMap );
  close( hSeq );
  close( hMon );
    
  return 0;
}
