/*
Test code for Monitor part of PCI-AER library, pciaerlib.o

testlibmon.c $Id: 36dc6af758c1dae9e53449273aea31ce27b97f3c $
*/


#include <fcntl.h>
#include <malloc.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/ioctl.h>

#include "pciaer.h"
#include "pciaerlib.h"

#define CHANNEL_BITMASK 0xF
#define TIME_LABELS	1
#define ARB_CONFIG	PCIAER_IOC_ARB_0_16
#define CLOCK_PERIOD	1

static int signalled = 0;

void signal_handler( int signum )
{
    signalled = 1;
}

int main( int argc, char *argv[] )
{
  int handle;
  int fifo_depth;
  pciaer_monitor_read_ae_t *pBuf;
  unsigned int nToRead;
  unsigned int pnRead;
  long lReadReturn;
  int j;

  //Open the Monitor
  if( PciaerMonOpen( 0, O_RDWR, &handle) )
    {
      perror( "open" );
      exit ( 1 );
    }
  //Monitor all channels
  if(PciaerMonSetChannelSel(handle, CHANNEL_BITMASK) !=0 )
    {
      perror( "PCIAER_IOC_SET_MON_CH_SEL" );
      exit( 1 );
    }
  //Set Arbiter configuration
  if(PciaerSetArbConfig(handle, ARB_CONFIG) != 0 )
    {
      perror( "PCIAER_IOC_SET_ARB_CONFIG" );
      exit( 1 );
    }
  //Enable/disable Time Flags
  if(PciaerMonSetTimeLabelFlag(handle, TIME_LABELS) != 0)
    {
      perror( "PCIAER_IOC_SET_MON_TIME_LBL_FLAG" );
      exit( 1 );
    }
  //Get Fifo Depth
  if(PciaerGetFifoDepth(handle, &fifo_depth) !=0)
    {
      perror( "PCIAER_IOC_GET_MON_FIFO_DEPTH" );
      exit( 1 );
    }
  //Set counter
  if(PciaerSetCounterPeriod(handle, CLOCK_PERIOD)!=0)
    {
      perror( "PCIAER_IOC_SET_CNTR_PERIOD" );
      exit( 1 );
    }
  //Reset FIFO
  if(PciaerResetFifo(handle)!=0)
    {    
      perror( "PCIAER_IOC_RST_MON_FIFO" );
      exit( 1 );
    }

  nToRead = TIME_LABELS ? ((fifo_depth + 2)/3) : fifo_depth;

  if ((pBuf = (pciaer_monitor_read_ae_t *)calloc(nToRead,sizeof(pciaer_monitor_read_ae_t))) == NULL )
    {
      printf("Memory allocation error\n");
      return 1;
    }

  while ( !signalled )
      {
	  lReadReturn = PciaerMonRead(handle, pBuf, nToRead, &pnRead);
	  if ( lReadReturn != 0L )
	      fprintf( stderr, "PciaerMonRead returned %08lX / %ldL\n", lReadReturn, lReadReturn );
	  if ( pnRead == 0 )
	      fprintf( stderr, "No events read\n" );
	  for( j=0; j<pnRead; j++ )
	      printf( "%u\t%u\n", pBuf[j].time_us, pBuf[j].ae );
      }
  free(pBuf);
  return 0;
}
