/*
Test code for Sequencer part of PCI-AER library, pciaerlib.o

testlibseq.c $Id: fa9afae033f0ee05c3375de46a47a5509898e5ff $
*/

#include <fcntl.h>
#include <malloc.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/ioctl.h>

#include "pciaer.h"
#include "pciaerlib.h"

int main( int argc, char *argv[] )
{
  int handle;
  //int map_handle;
  int clock_period_us = 1;
  struct timeval tv;
  unsigned int uc;
  pciaer_sequencer_write_ae_t *pBuf;
  unsigned int nToWrite=5000;
  unsigned int nWritten;
  int i;

#ifdef DEBUG
   printf("Start\n");
#endif  

  //Open the Sequencer
  if( PciaerSeqOpen( 0, O_RDWR, &handle ) != 0 )
    {
      perror( "open" );
      exit ( 1 );
    }
   //Set Sequencer Arbiter Channel
  if(PciaerSetSeqArbChannel( handle, 0 ) !=0)
    {
      perror( "PciaerSetSeqArbChannel" );
      exit( 1 );
    }
  //Set counter
  if(PciaerSetCounterPeriod(handle, clock_period_us)!=0)
    {
      perror( "PciaerSetCounterPeriod" );
      exit( 1 );
    }
  if ( PciaerResetCounterGetTime( handle, &tv ) != 0 )
      {
	  perror( "PciaerResetCounterGetTime" );
	  exit( 1 );
      }
#ifdef DEBUG
  printf( "Counter reset at %ld.%06ld\nSleeping ~1s...\n", tv.tv_sec, tv.tv_usec );
  sleep( 1 );
#endif

  if ( PciaerGetCounterValue( handle, &uc ) != 0 )
      {
	  perror( "PciaerGetCounterValue" );
	  exit( 1 );
      }

#ifdef DEBUG
  printf( "Counter value now ~%d\n", uc );
#endif
  
  //Open the Mapper
  /* map_handle = open( "/dev/aermap0", O_RDWR );
  if ( map_handle == -1 )
    {
      perror( "open" );
      exit( 1 );
    }
  if ( ioctl( map_handle, PCIAER_IOC_SET_MAP_AER_PROTOCOL, PCIAER_IOC_MAP_AER_P2P ) != 0 )
    {
      perror( "PCIAER_IOC_SET_MAP_AER_PROTOCOL" );
      exit( 1 );
      }
  close( map_handle );
  */

#ifdef DEBUG
   printf("Allocate memory\n");
#endif
  if ((pBuf = (pciaer_sequencer_write_ae_t *)calloc(nToWrite,sizeof(pciaer_sequencer_write_ae_t))) == NULL )
    {
      printf("Memory allocation error\n"); 
      return 1;
    }
#ifdef DEBUG
   printf("Memory allocated\n");
#endif
  for (i=0; i<nToWrite; i++)
    {
      pBuf[i].ae = 0; 
      pBuf[i].isi_us = 100;
    }
#ifdef DEBUG
  for (i=0; i<nToWrite; i++)
    { 
      printf("%d\t ae = %d\t isi = %d \n",i,pBuf[i].ae, pBuf[i].isi_us);
    }
   printf("Filled buffer\n");
#endif
  PciaerSeqWrite(handle,pBuf,nToWrite,&nWritten);
#ifdef DEBUG
   printf("Written %d events\n",nWritten);
#endif
  if (nWritten == 0)
    {
      printf("No events written\n");
      return 1;
    }
#ifdef DEBUG 
  printf("Free memory\n");
#endif
   free(pBuf);
#ifdef DEBUG
   printf("End\n");
#endif
   PciaerSeqClose(handle);
   return 0;
}
