/*
Test code for Mapper part of PCI-AER driver, pciaer.o

testmap.c $Id: 541a173c8619e9ea9c67e3f0d7d32f53367571d5 $
*/


#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/ioctl.h>

#define __USE_GNU
#include <sys/mman.h>

#include "pciaer.h"
#include "common.h"


unsigned short ae_list[] = { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10 };


void get_free_space( int handle )
{
    unsigned int free_space;
    if ( ioctl( handle, PCIAER_IOC_GET_MAP_FREE_SPACE, &free_space ) != 0 )
	{
	    perror( "PCIAER_IOC_GET_MAP_FREE_SPACE" );
	    exit( 1 );
	}
    printf( "Free space %u\n", free_space );
}


int main( int argc, char *argv[] )
{
  int handle;
  int seq_channel;
  int config;
  int channels;
  int fifo_flags;
  int state;
  int protocol;
  pciaer_mapping_t mapping;
  unsigned short ae;
  volatile unsigned int *p;
  int i;
  char chType;
  int j;

  handle = open( "/dev/aermap0", O_RDWR );
  if ( handle == -1 )
    {
      perror( "open" );
      exit( 1 );
    }

  print_version( handle );

  if ( argc == 2 && strcmp(argv[1], "--compact") == 0 )
  {
      printf( "Compacting...\t" );
      if ( ioctl( handle, PCIAER_IOC_MAP_COMPACT ) != 0 )
      {
          perror( "PCIAER_IOC_MAP_COMPACT" );
          exit( 1 );
      }
      printf( "Done\n" );
      exit( 0 );
  }

  test_lseek( handle );

  if ( ioctl( handle, PCIAER_IOC_GET_SEQ_ARB_CHANNEL, &seq_channel ) != 0 )
    {
      perror( "PCIAER_IOC_GET_SEQ_ARB_CHANNEL" );
      exit( 1 );
    }
  printf( "Sequencer arbiter channel %d\n", seq_channel );

  if ( ioctl( handle, PCIAER_IOC_GET_ARB_CONFIG, &config ) != 0 )
    {
      perror( "PCIAER_IOC_GET_ARB_CONFIG" );
      exit( 1 );
    }
  printf( "Arbiter config %d\n", config );

  if ( ioctl( handle, PCIAER_IOC_GET_MAP_CH_SEL, &channels ) != 0 )
    {
      perror( "PCIAER_IOC_SET_MAP_CH_SEL" );
      exit( 1 );
    }
  printf( "Channel selection bitmask 0x%08X\n", channels );

  if ( ioctl( handle, PCIAER_IOC_GET_MAP_FIFO_FLAGS, &fifo_flags ) != 0 )
    {
      perror( "PCIAER_IOC_GET_MAP_FIFO_FLAGS" );
      exit( 1 );
    }
  printf( "Fifo flags 0x%08X\n", fifo_flags );

  if ( ioctl( handle, PCIAER_IOC_GET_MAP_OUT_STATE, &state ) != 0 )
    {
      perror( "PCIAER_IOC_GET_MAP_OUT_STATE" );
      exit( 1 );
    }
  printf( "Mapper out state 0x%08X\n", state );

  if ( ioctl( handle, PCIAER_IOC_GET_MAP_OUT_CONFIG, &config ) != 0 )
    {
      perror( "PCIAER_IOC_GET_MAP_OUT_CONFIG" );
      exit( 1 );
    }
  printf( "Mapper out config %d\n", config );

  if ( ioctl( handle, PCIAER_IOC_GET_MAP_DEMUX_CONFIG, &config ) != 0 )
    {
      perror( "PCIAER_IOC_GET_MAP_DEMUX_CONFIG" );
      exit( 1 );
    }
  printf( "Demux config %d\n", config );

  if ( ioctl( handle, PCIAER_IOC_GET_MAP_AER_PROTOCOL, &protocol ) != 0 )
    {
      perror( "PCIAER_IOC_GET_MAP_AER_PROTOCOL" );
      exit( 1 );
    }
  printf( "Protocol %d\n", protocol );

  printf( "Disabling Mapper out\n" );
  if ( ioctl( handle, PCIAER_IOC_DISABLE_MAP_OUT, 1 ) != 0 )
    {
      perror( "PCIAER_IOC_DISABLE_MAP_OUT" );
      exit( 1 );
    }

  if ( ioctl( handle, PCIAER_IOC_GET_MAP_OUT_STATE, &state ) != 0 )
    {
      perror( "PCIAER_IOC_GET_MAP_OUT_STATE" );
      exit( 1 );
    }
  printf( "Mapper out state now 0x%08X\n", state );
  
/*   printf( "Enabling Mapper out\n" ); */
/*   if ( ioctl( handle, PCIAER_IOC_ENABLE_MAP_OUT, 1 ) != 0 ) */
/*     { */
/*       perror( "PCIAER_IOC_ENABLE_MAP_OUT" ); */
/*       exit( 1 ); */
/*     } */

/*   if ( ioctl( handle, PCIAER_IOC_GET_MAP_OUT_STATE, &state ) != 0 ) */
/*     { */
/*       perror( "PCIAER_IOC_GET_MAP_OUT_STATE" ); */
/*       exit( 1 ); */
/*     } */
/*   printf( "Mapper out state now 0x%08X\n", state ); */
  
  printf( "Clearing Mapper out\n" );
  if ( ioctl( handle, PCIAER_IOC_MAP_CLEAR ) != 0 )
    {
      perror( "PCIAER_IOC_MAP_CLEAR" );
      exit( 1 );
    }

  get_free_space( handle );

  if ( ioctl( handle, PCIAER_IOC_GET_MAP_OUT_STATE, &state ) != 0 )
    {
      perror( "PCIAER_IOC_GET_MAP_OUT_STATE" );
      exit( 1 );
    }
  printf( "Mapper out state now 0x%08X\n", state );

/*   if ( ioctl( handle, PCIAER_IOC_SET_MAP_MAPPING, 0xFFFFFFFF ) != -1 ) */
/*     { */
/*       printf( "PCIAER_IOC_SET_MAP_MAPPING with bad address failed to fail" ); */
/*       exit( 1 ); */
/*     } */
/*   perror( "PCIAER_IOC_SET_MAP_MAPPING correct error return" ); */

/*   mapping.source_ae = MAPPER_END_LABEL; */
/*   if ( ioctl( handle, PCIAER_IOC_SET_MAP_MAPPING, &mapping ) != -1 ) */
/*     { */
/*       printf( "PCIAER_IOC_SET_MAP_MAPPING with bad argument failed to fail" ); */
/*       exit( 1 ); */
/*     } */
/*   perror( "PCIAER_IOC_SET_MAP_MAPPING correct error return" ); */

/*   mapping.source_ae = 0; */
/*   mapping.dest_count = 0; */
/*   if ( ioctl( handle, PCIAER_IOC_SET_MAP_MAPPING, &mapping ) != 0 ) */
/*     { */
/*       perror( "PCIAER_IOC_SET_MAP_MAPPING" ); */
/*       exit( 1 ); */
/*     } */

/*   mapping.source_ae = 0; */
/*   mapping.dest_count = 1; */
/*   mapping.dest_ae = (unsigned short *)0xFFFFFFFF; */
/*   if ( ioctl( handle, PCIAER_IOC_SET_MAP_MAPPING, &mapping ) != -1 ) */
/*     { */
/*       printf( "PCIAER_IOC_SET_MAP_MAPPING with bad address failed to fail" ); */
/*       exit( 1 ); */
/*     } */
/*   perror( "PCIAER_IOC_SET_MAP_MAPPING correct error return" ); */

/*   mapping.source_ae = 0; */
/*   mapping.dest_count = 1; */
/*   mapping.dest_ae = &ae; */
/*   ae = MAPPER_END_LABEL; */
/*   if ( ioctl( handle, PCIAER_IOC_SET_MAP_MAPPING, &mapping ) != -1 ) */
/*     { */
/*       printf( "PCIAER_IOC_SET_MAP_MAPPING with bad argument failed to fail" ); */
/*       exit( 1 ); */
/*     } */
/*   perror( "PCIAER_IOC_SET_MAP_MAPPING correct error return" ); */

  mapping.source_ae = 0;
  mapping.dest_count = 1;
  mapping.dest_ae = &ae;
  ae = 1;
  if ( ioctl( handle, PCIAER_IOC_SET_MAP_MAPPING, &mapping ) != 0 )
    {
      perror( "PCIAER_IOC_SET_MAP_MAPPING" );
      exit( 1 );
    }
  printf( "Direct mapping succeeded\n" );

  get_free_space( handle );

  mapping.source_ae = 1;
  mapping.dest_count = 10; 
  mapping.dest_ae = ae_list;
  if ( ioctl( handle, PCIAER_IOC_SET_MAP_MAPPING, &mapping ) != 0 )
    {
      perror( "PCIAER_IOC_SET_MAP_MAPPING" );
      exit( 1 );
    }
  printf( "Indirect mapping succeeded\n" );

  get_free_space( handle );

  p = mmap( 0, sizeof(pciaer_mapper_sram_t), PROT_READ, MAP_SHARED, handle, 0 );
  if ( p == MAP_FAILED )
    {
      perror( "mmap" );
      exit( 1 );
    }
  
  if ( mprotect( (void *)p, sizeof(pciaer_mapper_sram_t), PROT_READ | PROT_WRITE ) == 0 )
      {
	  fprintf( stderr, "mprotect erroneously granted PROT_WRITE\n" );
	  exit( 1 );
      }
  else if ( errno != EACCES )
      {
	  perror( "mprotect" );
	  exit( 1 );
      }
  printf( "mprotect correctly failed to grant PROT_WRITE\n" );

  if ( mremap( (void *)p, sizeof(pciaer_mapper_sram_t), sizeof(pciaer_mapper_sram_t) + getpagesize(), 0 ) != (void *)-1 )
      {
	  fprintf( stderr, "mremap erroneously allowed mapped memory to be expanded\n" );
	  exit( 1 );
      }
  else if ( errno != EFAULT )
      {
	  perror( "mremap" );
	  exit( 1 );
      }
  printf( "mremap correctly failed to expand mapped memory\n" );
  
  printf( "Disabling Mapper out\n" );
  if ( ioctl( handle, PCIAER_IOC_DISABLE_MAP_OUT, 1 ) != 0 )
    {
      perror( "PCIAER_IOC_DISABLE_MAP_OUT" );
      exit( 1 );
    }

  if ( ioctl( handle, PCIAER_IOC_GET_MAP_OUT_STATE, &state ) != 0 )
    {
      perror( "PCIAER_IOC_GET_MAP_OUT_STATE" );
      exit( 1 );
    }
  printf( "Mapper out state now 0x%08X\n", state );

  for ( i = 0; i < PCIAER_MAPPER_PTR_TBL_LENGTH; i++ )
  {
    switch ( p[ i ] & MAPPER_PTR_TYPE_MASK )
      {
      case MAPPER_PTR_TYPE_NOT_USED_2:
	chType = '2';
	break;
      case MAPPER_PTR_TYPE_INDIRECT:
	chType = 'I';
	break;
      case MAPPER_PTR_TYPE_NOT_USED_1:
	chType = '1';
	break;
      case MAPPER_PTR_TYPE_DIRECT:
	chType = 'D';
	break;
      }
    printf( "[0x%04X]: 0x%08X %c", i, p[ i ], chType );
    if ( chType == 'D' )
      printf( " 0x%04X", p[ i ] & MAPPER_PTR_DIRECT_MASK );
    if ( chType == 'I' )
      {
	j = p[ i ] & MAPPER_PTR_INDIRECT_MASK;
	printf( " 0x%06X\n[0x%06X]:", j, j );
	do
	  printf( " 0x%08X", p[ j ] );
	while ( (p[ j++ ] & MAPPER_PTR_DIRECT_MASK) != MAPPER_END_LABEL );
      }
    printf( "\n" );
  }

  if ( munmap( (void *)p, sizeof(pciaer_mapper_sram_t) ) != 0 )
    {
      perror( "munmap" );
      exit( 1 );
    }
  
  close( handle );
    
  return 0;
}
